@@ -4,10 +4,10 @@ use strict;
 use warnings;
 
 use HTTP::Tiny;
-use JSON::PP;
+use JSON;
 use Data::CosineSimilarity;
 
-our $VERSION = '0.1_2';
+our $VERSION = '0.1_3';
 $VERSION = eval $VERSION;
 
 my $http = HTTP::Tiny->new;
@@ -25,8 +25,6 @@ sub new {
     
     $attr{'model'}      = 'text-embedding-ada-002' unless $attr{'model'};
     
-    $attr{'comparator'} = $class->_make_vector($attr{'comparator'}) if exists $attr{'comparator'};
-    
     return bless \%attr, $class;
 }
 
@@ -55,6 +53,7 @@ sub error {
 # Header for calling OpenAI
 sub _get_header_openai {
     my $self = shift;
+    $self->{'key'} = '' unless defined $self->{'key'};
     return {
          'Authorization' => 'Bearer ' . $self->{'key'},
          'Content-type'  => 'application/json'
@@ -82,7 +81,7 @@ sub _get_header_openai {
  # Currently hard coded to OpenAI
  
  # Added purely for testing - IGNORE!
- sub test {
+ sub _test {
      my $self = shift;
 #    return $self->{'api'};
      return $header{$self->{'api'}};
@@ -90,7 +89,7 @@ sub _get_header_openai {
  
  # Return Embedding as a CSV string
  sub embedding {
-     my ($self, $text) = @_;
+     my ($self, $text, $verbose) = @_;
      
      my $response = $self->_get_embedding($text);
      if ($response->{'success'}) {
@@ -98,12 +97,13 @@ sub _get_header_openai {
          return join (',', @{$embedding->{'data'}[0]->{'embedding'}});
      }
      $self->{'error'} = 'HTTP Error - ' . $response->{'reason'};
-     return $response;
+     return $response if defined $verbose;
+     return undef;
  }
  
  # Return Embedding as an array
  sub raw_embedding {
-     my ($self, $text) = @_;
+     my ($self, $text, $verbose) = @_;
      
      my $response = $self->_get_embedding($text);
      if ($response->{'success'}) {
@@ -111,12 +111,14 @@ sub _get_header_openai {
          return @{$embedding->{'data'}[0]->{'embedding'}};
      }
      $self->{'error'} = 'HTTP Error - ' . $response->{'reason'};
-     return $response;
+     return $response if defined $verbose;
+     return undef;
  }
  
  # Return Test Embedding
  sub test_embedding {
      my ($self, $text, $dimension) = @_;
+     $self->{'error'} = '';
      
      $dimension = 1536 unless defined $dimension;
      
@@ -143,12 +145,17 @@ sub _get_header_openai {
     return \%vector;
 }    
  
-# Set a vector to compare
+# Return a comparator to compare to a set vector
 sub comparator {
-    my ($self, $embed) = @_;
+    my($self, $embed) = @_;
+    $self->{'error'} = '';
     
-    $self->{'comparator'} = $self->_make_vector($embed);
-    return;
+    my $vector1 = $self->_make_vector($embed);
+    return sub {
+        my($embed2) = @_;
+        my $vector2 = $self->_make_vector($embed2);
+        return $self->_compare_vector($vector1, $vector2);
+    };
 }
 
 # Compare 2 Embeddings
@@ -173,6 +180,12 @@ sub compare {
         return;
     }
     
+    return $self->_compare_vector($vector1, $vector2);
+}
+
+# Compare 2 Vectors    
+sub _compare_vector {
+    my ($self, $vector1, $vector2) = @_;
     my $cs = Data::CosineSimilarity->new;
     $cs->add( label1 => $vector1 );
     $cs->add( label2 => $vector2 );
@@ -183,6 +196,8 @@ sub compare {
 
 __END__
 
+=encoding utf8
+
 =head1 NAME
 
 AI::Embedding - Perl module for working with text embeddings using various APIs
@@ -205,9 +220,10 @@ This is currently a development release and may contain bugs!
     my $csv_embedding  = $embedding->embedding('Some sample text');
     my $test_embedding = $embedding->test_embedding('Some sample text');
     my @raw_embedding  = $embedding->raw_embedding('Some sample text');
-    $embedding->comparator($csv_embedding2);
+    
+    my $cmp = $embedding->comparator($csv_embedding2);
 
-    my $similarity = $embedding->compare($csv_embedding1);
+    my $similarity = $cmp->($csv_embedding1);
     my $similarity_with_other_embedding = $embedding->compare($csv_embedding1, $csv_embedding2);
 
 =head1 DESCRIPTION
@@ -220,7 +236,9 @@ An Embedding is a multi-dimensional vector representing the meaning of a piece o
 
 =head2 Comparator
 
-Embeddings are used to compare similarity of meaning between two passages of text.  A typical work case is to store a number of pieces of text (e.g. articles or blogs) in a database and compare each one to some user supplied search text.  L<AI::Embedding> provides a C<compare> method to either compare two Embeddings or one Embedding to a previously supplied C<compatator>.  The C<comparator> can either be set when the object is constructed or by using the B<comparator> method.  However, it is not usually available at the time tyhe object is created so will generally be supplied with the B<comparator> method.
+Embeddings are used to compare similarity of meaning between two passages of text.  A typical work case is to store a number of pieces of text (e.g. articles or blogs) in a database and compare each one to some user supplied search text.  L<AI::Embedding> provides a C<compare> method compare two Embeddings.
+
+Alternatively, the C<comparator> method can be called with one Embedding.  The C<comparator> returns a reference to a method that takes a single Embedding to be compared to the Embedding from which the Comparator was created.
 
 When comparing multiple Embeddings to the same Embedding (such as search text) it is faster to use a C<comparator>.
 
@@ -232,9 +250,8 @@ When comparing multiple Embeddings to the same Embedding (such as search text) i
         api         => 'OpenAI', 
         key         => 'your-api-key',
         model       => 'text-embedding-ada-002',
-        comparator  => $search_string,
     );
-    
+
 Creates a new AI::Embedding object. It requires the 'key' parameter. The 'key' parameter is the API key provided by the service provider and is required.
 
 Parameters:
@@ -253,10 +270,6 @@ C<api> - The API to use.  Currently only 'OpenAI' is supported and this is the d
 
 C<model> - The language model to use.  Defaults to C<text-embedding-ada-002> - see L<OpenAI docs|https://platform.openai.com/docs/guides/embeddings/what-are-embeddings>
 
-=item *
-
-C<comparator> - Set the C<comparator> - see L</"Comparator">
-
 =back
 
 =head1 METHODS
@@ -271,13 +284,13 @@ Returns the last error message or an empty string if B<success> returned true
 
 =head2 embedding
 
-    my $csv_embedding = $embedding->embedding('Some text passage');
+    my $csv_embedding = $embedding->embedding('Some text passage', [verbose]);
 
 Generates an embedding for the given text and returns it as a comma-separated string. The C<embedding> method takes a single parameter, the text to generate the embedding for.
 
 Returns a (rather long) string that can be stored in a C<TEXT> database field.
 
-If the method call fails it sets the L</"error"> message and returns the complete L<HTTP::Tiny> response object.
+If the method call fails it sets the L</"error"> message and returns C<undef>.  If the optional C<verbose> parameter is true, the complete L<HTTP::Tiny> response object is also returned to aid with debugging issues when using this module.
 
 =head2 raw_embedding
 
@@ -287,31 +300,42 @@ Generates an embedding for the given text and returns it as an array. The C<raw_
 
 It is not normally necessary to use this method as the Embedding will almost always be used as a single homogeneous unit.
 
-If the method call fails it sets the L</"error"> message and returns the complete L<HTTP::Tiny> response object.
+If the method call fails it sets the L</"error"> message and returns C<undef>.  If the optional C<verbose> parameter is true, the complete L<HTTP::Tiny> response object is also returned to aid with debugging issues when using this module.
 
-=head2 raw_embedding
+=head2 test_embedding
 
-    my @raw_embedding = $embedding->raw_embedding('Some text passage', $dimensions);
-    
-Used for testing code without making a chargable call to the API.  Returns a random embedding.  Both parameters are optional.  If a text string is provided, the returned embedding will always be the same random embedding otherwise it will random and different every time.  C<dimension> controls the number of elements of the returned CSV string.  If omitted, the string will have the C<text-embedding-ada-002> default of 1536 elements.
+    my @test_embedding = $embedding->test_embedding('Some text passage', $dimensions);
+
+Used for testing code without making a chargeable call to the API.
+
+Returns a random embedding.  Both parameters are optional.  If a text string is provided, the returned embedding will always be the same random embedding otherwise it will be random and different every time.  The C<dimension> parameter controls the number of elements of the returned CSV string.  If omitted, the string will have the C<text-embedding-ada-002> default of 1536 elements.
 
 =head2 comparator
 
     $embedding->comparator($csv_embedding2);
 
-Sets a vector as a C<comparator> for future comparisons. The B<comparator> method takes a single parameter, the comma-separated embedding string to use as the comparator.
+Sets a vector as a C<comparator> for future comparisons and returns a reference to a method for using the C<comparator>.
+
+The B<comparator> method takes a single parameter, the comma-separated Embedding string to use as the comparator.
+
+The following two are functionally equivalent.  However, where multiple Embeddings are to be compared to a single Embedding, using a L<Comparator> is significantly faster.
+
+    my $similarity = $embedding->compare($csv_embedding1, $csv_embedding2);
+
+
+    my $cmp = $embedding->comparator($csv_embedding2);
+    my $similarity = $cmp->($csv_embedding1);
 
 See L</"Comparator">
 
+The returned method reference returns the cosine similarity between the Embedding used to call the C<comparator> method and the Embedding supplied to the method reference.  See L<compare> for an explanation of the cosine similarity.
+
 =head2 compare
 
-    my $similarity = $embedding->compare($csv_embedding1);
     my $similarity_with_other_embedding = $embedding->compare($csv_embedding1, $csv_embedding2);
 
 Compares two embeddings and returns the cosine similarity between them. The B<compare> method takes two parameters: $csv_embedding1 and $csv_embedding2 (both comma-separated embedding strings). 
 
-If only one parameter is provided, it is compared with the previously set C<comparator>.
-
 Returns the cosine similarity as a floating-point number between -1 and 1, where 1 represents identical embeddings, 0 represents no similarity, and -1 represents opposite embeddings.
 
 The absolute number is not usually relevant for text comparision.  It is usually sufficient to rank the comparison results in order of high to low to reflect the best match to the worse match.
@@ -352,7 +376,9 @@ L<https://metacpan.org/release/AI::Embedding>
 
 =head1 ACKNOWLEDGEMENTS
 
-Thanks to the help and support provided by members of Perl Monks L<https://perlmonks.org/>
+Thanks to the help and support provided by members of Perl Monks L<https://perlmonks.org/>.
+
+Especially L<KCOTT|https://metacpan.org/author/KCOTT> for assistance with unit tests and L<hv|https://perlmonks.org/index.pl?node_id=209409> for suggesting the current C<comparator> implementaion.
 
 =head1 COPYRIGHT AND LICENSE
 
